list(
  # Adjustment name ----
  Name = "Pre-process TESSy input data",

  # Input parameters to the adjustment function ----
  Parameters = list(
  ),

  # Names of packages that must be made available to the adjustment function ----
  RequiredPackageNames = c(),

  ## Adjustment function ----
  AdjustmentFunction = function(inputData, parameters) {
    # Rename columns
    oldColNames <- c("recordid", "age", "regionoforigin", "countryofnationality", "countryofbirth",
                     "reportingcountry", "transmission", "dateofdiagnosisyear", "cd4_num",
                     "gender", "dateofaidsdiagnosisyear", "dateofnotificationyear",
                     "dateofnotificationquarter", "dateofdiagnosisquarter", "firstcd4dateyear",
                     "firstcd4datequarter", "firstcd4dateisodate", "dateofdeathyear",
                     "dateofdeathquarter", "dateofdeathdate", "outcome", "placeofnotification",
                     "placeofresidence")
    newColNames <- c("RecordId", "Age", "RegionOfOrigin", "CountryOfNationality", "CountryOfBirth",
                     "ReportingCountry", "Transmission", "DateOfDiagnosisYear", "CD4_num",
                     "Gender", "DateOfAIDSdiagnosisYear", "DateOfNotificationYear",
                     "DateOfNotificationQuarter", "DateOfDiagnosisQuarter", "FirstCD4DateYear",
                     "FirstCD4DateQuarter", "FirstCD4DateISOdate", "DateOfDeathYear",
                     "DateOfDeathQuarter", "DateOfDeathDate", "Outcome", "PlaceOfNotification",
                     "PlaceOfResidence")
    setnames(inputData, oldColNames, newColNames)

    # Keep only required variables
    outputData <- inputData[, ..newColNames]

    # Convert all strings to upper case
    colClasses <- sapply(outputData, class)
    charColNames <- newColNames[colClasses == "character"]
    outputData[, (charColNames) := lapply(.SD, toupper), .SDcols = charColNames]

    # Replace UNKs and BLANKS with NAs
    is.na(outputData) <- outputData == "UNK"
    is.na(outputData) <- outputData == ""

    # Drop unused levels (prior UNKs)
    outputData <- droplevels(outputData)

    # Create Migr variable 1, 2, 3...
    outputData[, Migr := factor(NA, levels = c("Reporting Country", "Other Country", "SSA"))]
    # ...based on RegionOfOrigin if not NA
    outputData[is.na(Migr) & !is.na(RegionOfOrigin),
               Migr := ifelse(RegionOfOrigin == "REPCOUNTRY", 1L,
                              ifelse(!RegionOfOrigin %in% "SUBAFR", 2L,
                                     3L))]
    # ...based on CountryOfBirth if not NA
    outputData[is.na(Migr) & !is.na(CountryOfBirth),
               Migr := ifelse(CountryOfBirth == ReportingCountry, 1L,
                              ifelse(!CountryOfBirth %in% countryCodes, 2L,
                                     3L))]
    # ...based on CountryOfNationality if not NA
    outputData[is.na(Migr) & !is.na(CountryOfNationality),
               Migr := ifelse(CountryOfNationality == ReportingCountry, 1L,
                              ifelse(!CountryOfNationality %in% countryCodes, 2L,
                                     3L))]

    # Transform CD4
    outputData[, SqCD4 := sqrt(CD4_num)]

    # AIDS close to diagnosis
    outputData[, AIDS := factor(ifelse(!is.na(DateOfAIDSdiagnosisYear),
                                       DateOfAIDSdiagnosisYear - DateOfDiagnosisYear <= 1L,
                                       FALSE),
                                labels = c("AIDS-No", "AIDS-Yes"))]

    # Transform Transmission
    outputData[, Transmission := factor(Transmission)]

    # Drop no longer needed vars
    outputData[, ":="(
      RegionOfOrigin = NULL,
      CountryOfNationality = NULL,
      CountryOfBirth = NULL,
      DateOfAIDSdiagnosisYear = NULL
    )]

    return(outputData)
  }
)
